"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const postcss_1 = require("postcss");
const postcss_value_parser_1 = __importDefault(require("postcss-value-parser"));
const utils_1 = require("./utils");
const config_key_1 = require("../../lib/config-key");
function* getParents(node) {
    let { parent } = node;
    while (parent) {
        yield parent;
        parent = parent.parent;
    }
}
function buildNamespaceFromParents(componentAssociations, node) {
    const namespaceFragments = [];
    for (const parent of getParents(node)) {
        if (!componentAssociations.has(parent))
            continue;
        // @todo https://github.com/microsoft/TypeScript/issues/13086
        // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
        const currentNamespaceFragment = componentAssociations.get(parent);
        if (config_key_1.isValidRootConfigKey(currentNamespaceFragment)) {
            namespaceFragments.push(config_key_1.resolveRootConfigKey(currentNamespaceFragment));
            break;
        }
        namespaceFragments.push(currentNamespaceFragment);
    }
    return namespaceFragments.reverse().join('.');
}
function namespaceConfigKey(namespace, key) {
    if (config_key_1.isValidRootConfigKey(key))
        return config_key_1.resolveRootConfigKey(key);
    return `${namespace}.${key}`;
}
function transformContextAtRule(atRule, namespace) {
    const configKey = atRule.params.length === 0 ? 'context' : atRule.params;
    utils_1.assertValidConfigKey(configKey, atRule);
    atRule.params = namespaceConfigKey(namespace, configKey);
}
function transformDecl(decl, namespace, keyword = 'cfg') {
    const originalValue = decl.value;
    decl.value = postcss_value_parser_1.default(originalValue)
        .walk(node => {
        if (node.type !== 'function' || node.value !== keyword)
            return;
        const keyNode = utils_1.getKeyNodeFromFunctionNode(decl, node, keyword);
        keyNode.value = namespaceConfigKey(namespace, keyNode.value);
        node.nodes = [keyNode];
    })
        .toString();
}
function transformOne(container, component, keyword = 'cfg') {
    container.walkDecls(decl => {
        // Only parse and transform, if the function, e.g. `cfg(`, is present.
        if (!decl.value.includes(`${keyword}(`))
            return;
        transformDecl(decl, component, keyword);
    });
    container.walkAtRules('context', atRule => transformContextAtRule(atRule, component));
}
function transformMany(container, componentAssociations, keyword = 'cfg') {
    container.walkDecls(decl => {
        // Only parse and transform, if the function, e.g. `cfg(`, is present.
        if (!decl.value.includes(`${keyword}(`))
            return;
        const parentNamespace = buildNamespaceFromParents(componentAssociations, decl);
        if (!parentNamespace)
            return;
        transformDecl(decl, parentNamespace, keyword);
    });
    container.walkAtRules('context', atRule => {
        const parentNamespace = buildNamespaceFromParents(componentAssociations, atRule);
        transformContextAtRule(atRule, parentNamespace);
    });
}
/**
 * Expands `@component` at-rule shortcuts.
 *
 * ```css
 * @component button;
 *
 * .primary-button {
 *   @component primary;
 *   background: cfg('background');
 *   font-size: cfg('/typo.sizes.medium');
 * }
 *
 * .something-else {
 *   @component /typo.sizes;
 *  font-size: cfg('medium');
 * }
 * ```
 *
 * ```css
 * .primary-button {
 *   background: cfg('button.primary.background');
 *   font-size: cfg('typo.sizes.medium');
 * }
 *
 * .something-else {
 *   font-size: cfg('typo.sizes.medium');
 * }
 * ```
 */
exports.default = postcss_1.plugin('postcss-ember-makeup:expand-component-shortcut', () => {
    const keyword = 'component';
    return root => {
        utils_1.assertNoDoubleColonAtRule(keyword, root);
        const componentAssociations = new Map();
        root.walkAtRules(keyword, atRule => {
            const { params: configKey, parent } = atRule;
            utils_1.assertValidConfigKey(configKey, atRule);
            if (componentAssociations.has(parent)) {
                throw atRule.error(`You can only specify a '@component' once per container.`);
            }
            componentAssociations.set(parent, configKey);
            atRule.remove();
        });
        switch (componentAssociations.size) {
            case 0:
                return;
            case 1:
                transformOne(...[...componentAssociations.entries()][0]);
                break;
            default:
                transformMany(root, componentAssociations);
        }
    };
});
